@@ -44,12 +44,6 @@
 #define ARMA_INCFILE_WRAP(x) <x>
 
 
-#if (__cplusplus >= 201103L)
-  #undef  ARMA_USE_CXX11
-  #define ARMA_USE_CXX11
-#endif
-
-
 #if defined(ARMA_USE_CXX11)
   #undef  ARMA_USE_U64S64
   #define ARMA_USE_U64S64
@@ -69,6 +63,8 @@
     #define ARMA_HAVE_SNPRINTF
     #define ARMA_HAVE_ISFINITE
     #define ARMA_HAVE_LOG1P
+    #define ARMA_HAVE_ISINF
+    #define ARMA_HAVE_ISNAN
   #endif
 #endif
 
@@ -121,7 +117,7 @@
   
   #define ARMA_GCC_VERSION (__GNUC__ * 10000 + __GNUC_MINOR__ * 100 + __GNUC_PATCHLEVEL__)
   
-  #if (ARMA_GCC_VERSION < 40200)
+  #if (ARMA_GCC_VERSION < 40200) && !defined(__INTEL_COMPILER)
     #error "*** Need a newer compiler ***"
   #endif
   
@@ -155,14 +151,11 @@
   
   #define ARMA_HAVE_ALIGNED_ATTRIBUTE
   
-  #if defined(__GXX_EXPERIMENTAL_CXX0X__)
-    #undef  ARMA_USE_CXX11
-    #define ARMA_USE_CXX11
-  #endif
-  
   #if defined(ARMA_USE_CXX11)
-    #if (ARMA_GCC_VERSION < 40700) && !defined(__clang__)
-      #pragma message ("Your C++ compiler is in C++11 mode, but it has incomplete support for C++11 features")
+    #if (ARMA_GCC_VERSION < 40800) && !defined(__clang__)
+      #pragma message ("WARNING: your C++ compiler is in C++11 mode, but it has incomplete support for C++11 features; if something breaks, you get to keep all the pieces")
+      #pragma message ("WARNING: to forcefully prevent Armadillo from using C++11 features, #define ARMA_DONT_USE_CXX11 before #include <armadillo>")
+      #define ARMA_DONT_USE_CXX11_CHRONO
     #endif
   #endif
   
@@ -216,6 +209,16 @@
     #error "*** Need a newer compiler ***"
   #endif
   
+  #if (_MSC_VER < 1700)
+    #pragma message ("WARNING: your C++ compiler is outdated and has incomplete support for the C++ standard; if something breaks, you get to keep all the pieces")
+  #endif
+  
+  #if defined(ARMA_USE_CXX11)
+    #if (_MSC_VER < 1800)
+      #pragma message ("WARNING: your C++ compiler is in C++11 mode, but it has incomplete support for C++11 features; if something breaks, you get to keep all the pieces")
+    #endif
+  #endif
+  
   #undef  ARMA_SIMPLE_LOOPS
   #define ARMA_SIMPLE_LOOPS
   
@@ -287,13 +290,15 @@
 #endif
 
 
-#if defined(__CUDACC__)
+#if defined(__NVCC__)
   
   #undef ARMA_HAVE_SNPRINTF
   #undef ARMA_HAVE_ISFINITE
   #undef ARMA_HAVE_LOG1P
   #undef ARMA_HAVE_TR1
   
+  #undef  arma_noinline
+  #define arma_noinline
 #endif
 
 
